package com.xydl.cac.service.impl;

import com.xydl.cac.entity.ModevType;
import com.xydl.cac.entity.NiecPoint;
import com.xydl.cac.entity.NiecSensor;
import com.xydl.cac.model.ColumnModel;
import com.xydl.cac.model.ConditionModel;
import com.xydl.cac.model.OnePage;
import com.xydl.cac.repository.ModevTypeRepository;
import com.xydl.cac.service.DataService;
import com.xydl.cac.util.DateUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

@Service
@Slf4j
@Transactional(rollbackFor = Exception.class)
public class DataServiceImpl implements DataService {

    private String schema;

    @Resource
    private JdbcTemplate jdbcTemplate;
    @Resource
    ModevTypeRepository modevTypeRepository;

    @Override
    public List<ColumnModel> getDataTables() throws Exception {
        List<ModevType> typeList = modevTypeRepository.findAll();
        List<ColumnModel> tables = new ArrayList<>();
        String sql = "SHOW TABLES";
        List<String> list = jdbcTemplate.queryForList(sql, String.class);
        for (String name : list) {
            if (name.startsWith("data_")) {
                ColumnModel item = new ColumnModel();
                item.setName(name);
                for (ModevType type : typeList) {
                    if (name.equals(type.getTablename())) {
                        item.setComment(type.getMc());
                        break;
                    }
                }
                tables.add(item);
            }
        }
        return tables;
    }

    @Override
    public List<ColumnModel> getDataTableColumns(String tableName) throws Exception {
        if (StringUtils.isBlank(schema)) {
            this.getDatabase();
        }

        List<ColumnModel> result = new ArrayList<>();
        String sql = "SELECT COLUMN_NAME name, COLUMN_COMMENT comment FROM information_schema.columns WHERE TABLE_NAME='"
                + tableName + "' AND TABLE_SCHEMA='" + schema + "'";
        List<ColumnModel> list = jdbcTemplate.query(sql, new BeanPropertyRowMapper<>(ColumnModel.class));
        for (ColumnModel item : list) {
            String col = item.getName();
            if (!col.equals("id") && !col.equals("eqmid")
                    && !col.equals("acquisitionTime") && !col.equals("d_time")
                    && !col.equals("create_time") && !col.equals("update_time")
                    && !col.equals("isupload")) {
                result.add(item);
            }
        }
        return result;
    }

    private void getDatabase() {
        String sql = "select database()";
        List<String> list = jdbcTemplate.queryForList(sql, String.class);
        schema = list.get(0);
    }

    @Override
    public OnePage<Map<String, Object>> getDate(NiecSensor sensor, List<NiecPoint> points, ConditionModel model) {
        OnePage result = new OnePage();

        String sqlFrom = " FROM " + sensor.getTableName();
        String sqlWhere = " WHERE " + sensor.getDevidFieldName() + "=" + sensor.getDevId();
        if (model.getStartTime() != null) {
            sqlWhere = sqlWhere + " AND " + sensor.getDtimeFieldName() + " >= '" + DateUtil.format(model.getStartTime()) + "'";
        }
        if (model.getEndTime() != null) {
            sqlWhere = sqlWhere + " AND " + sensor.getDtimeFieldName() + " < '" + DateUtil.format(model.getEndTime()) + "'";
        }
        String sqlOrder = " ORDER BY " + sensor.getDtimeFieldName() + " DESC";
        String sqlLimit = "";
        if (model.getPageNum() != null && model.getPageSize() != null) {
            result.setPageNum(model.getPageNum());
            result.setPageSize(model.getPageSize());
            sqlLimit = String.format(" LIMIT %d,%d", (model.getPageNum() - 1) * model.getPageSize(), model.getPageSize());
        }

        String sqlSelect = "SELECT DATE_FORMAT(" + sensor.getDtimeFieldName() + ", '%Y-%m-%d %H:%i:%s') as acquisitionTime";
        for (NiecPoint point : points) {
            if (StringUtils.isNotBlank(point.getField())) {
                sqlSelect = sqlSelect + ", " + point.getField();
            }
        }

        String sqlCount = "SELECT count(*)" + sqlFrom + sqlWhere;
        String sql = sqlSelect + sqlFrom + sqlWhere + sqlOrder + sqlLimit;
        log.info(sql);

        Long count = jdbcTemplate.queryForObject(sqlCount, Long.class);
        List<Map<String, Object>> content = jdbcTemplate.queryForList(sql);

        result.setTotal(count);
        result.setContent(content);
        return result;
    }
}
